#!/usr/bin/env python3
""" Assignment 6: sequence-to-sequence networks
    See <https://snlp2019.github.io/a6/> for instructions.

    **Please insert your name(s), and the names of people or links to
    the resources that you got help for your solution.**
"""

# The following are provided for your convenience.
# Note that we use the tokenized data from the input file.
# That is the input words are represented with space-separated
# segments (IPA segments may be composed of multiple Unicode
# characters).
#
# The code is not well tested. There may be bugs.

import numpy as np
import csv


def read_data(filename, source, target):
    d = dict()
    with open(filename, 'r') as fp:
        tsv = csv.DictReader(fp, delimiter='\t')
        for row in tsv:
            concept = row['Concept_ID']
            lang = row['Language_ID']
            word = row['IPA']
            if concept not in d: d[concept] = dict()
            d[concept][lang] = word
    word_pairs = []
    for concept, word in d.items():
        if source in word and target in word:
            word_pairs.append((word[source], word[target]))
    return word_pairs

class CharEncoder:
    """ Utility to transform character sequences to numbers and back.
    """
    def __init__(self):
        self.char2int = {'PAD': 0, 'UNK': 1, 'EOS': 2}
        self.int2char = ['', '�', '']
        self.pad = self.char2int['PAD']
        self.eos = self.char2int['EOS']
        self.unk = self.char2int['UNK']

    def fit(self, word_list):
        self.max_len = 0
        for word in word_list:
            w = word.split()
            if len(w) > self.max_len:
                self.max_len = len(w)
            for ch in w:
                if ch not in self.char2int:
                    self.char2int[ch] = len(self.char2int)
                    self.int2char.append(ch)

    def transform(self, word_list, max_len=None,
            pad_side='left', pad=0):
        if max_len is None:
            max_len = self.max_len
        assert max_len != 0
        encoded = []
        for word in word_list:
            wint = []
            chars = word.split()
            for i in range(min(len(chars), max_len)):
                wint.append(self.char2int.get(chars[i], self.unk))
            wint.append(self.eos)
            if len(chars) < max_len:
                if pad_side == 'left':
                    wint = (max_len - len(chars)) * [pad] + wint
                else: # right
                    wint += (max_len - len(chars)) * [pad] 
            encoded.append(wint)
        return encoded

    def inv_transform(self, encoded_word):
        word = []
        for i in encoded_word.astype('int'):
            if i == self.eos: break
            word.append(self.int2char[i])
        return ' '.join(word)



def train_seq2seq(x, y,
        emb_dim = 64, enc_dim = 256, dropout=0.5, eos_sym=2, max_epoch=100):
    """ 6.2: Build, train and return a squence-to-sequence model
    Arguments:
    x           Input, a numpy array with shape
                    (n_words, max_seq_len)
    y           Output a numpy array with shape
                    (n_words, max_seq_len)
    emb_dim     Dimension of the embeddings 
    enc_dim     Dimension encoder hidden units (bot for forward and
                    backward RNNs)
    """
    pass

def predict(model, words, eos_sym=2):
    """ 6.3: Given a sequence of words, predict the most likely target words

    Given a list of input words, for each word,
    predict the output of the decoder character-by-character
    until end-of-sequence (eos) symbol is predicted.
    Return predictions.

    The function returns the encoded target words.

    Arguments:

    model       A trained seq2seq model
    word        Input word (encoded): numpy array with shape (seq_len,)
    eos_sym     End of sequence symbol
    """
    pass

def evaluate(gold, predicted):
    """ 6.4: calculate and return mean edit distance between gold and predicted

    Given gold-standard and predicted words (sequences of IPA symbols)
    return the mean edit distance
    """
    pass

if __name__ == "__main__":

    # Optional exercise: take the source and target language,
    # the input file, and train/test split rate from the command line.
    # You can also include parameters of the seq2seq network.

    word_pairs = read_data('northeuralex-cldf.csv', 'deu', 'nld')

    # Exercise 6.1 and rest of your testing should go here.
    # Expected flow here is as follows (the exercise is only the first
    # item)
    #
    # - split the data set (exercise 6.1)
    # - encode them using the Encoder provided above (use the same
    #   encoder for both source and target languages)
    # - build and train the network
    # - obtain predictions on the test set
    # - evaluate, and print the score
